# -*- coding: utf-8 -*-
from odps.models import Schema, Column, Partition

from plbrdidge.core.config import conf, df2dw_conf
from plbrdidge.core.tqs import Tqs
from plbrdidge.utils import utils
from plbrdidge.utils.dataframe_utils import DataFrameUtils
from plbrdidge.utils.duration import Duration
from plbrdidge.utils.logger import logger


class Multithreading:
    def __init__(self):
        self.prefix = ''  # nothing
        self.dfu = DataFrameUtils() if conf['df'] is None else conf['df']
        self.tqs = Tqs()

    def init(self, input_tables, prefix=None, drop=True):
        self.prefix = '' if prefix is None else prefix
        for input_table in input_tables:
            if not drop and self.dfu.exist_table(input_table):
                continue
            result = self.tqs._run_sql(sql=f'DESC {input_table};', table=input_table, return_flag=True)
            columns, partitions, cf = [], [], True
            for i, (col_name, data_type, comment, _) in enumerate(result):
                if i == 0:  # col_name, data_type, comment, _
                    continue
                if col_name.startswith('#'):
                    cf = False
                    continue
                if cf:
                    columns.append(Column(name=col_name, type=data_type, comment=comment))
                else:
                    partitions.append(Partition(name=col_name, type=data_type, comment=comment))
            col_len = len(columns) - len(partitions)
            self._generate_table(table=input_table, columns=columns[0:col_len], partitions=partitions, drop=drop)

    def _run_sql(self, sql_list):
        duration = Duration()
        inst_list = []
        for i in range(len(sql_list)):
            logger.info(f'[{i + 1}|{len(sql_list)}],{sql_list[i]}')
            inst = self.dfu._client().run_sql(sql_list[i], priority=2)
            logger.info(inst.get_logview_address())
            inst_list.append(inst)
        for inst in inst_list:
            inst.wait_for_completion()
        for inst in inst_list:
            utils.instance_status(inst)
        logger.info(f'{self.__class__.__name__}:run_sql:{len(sql_list)} sql {duration.minute()}')

    def async_df(self, df, out_table, drop=True, partition_list=None, tqs=True):
        duration = Duration()
        # generate tables with columns and partitions
        columns, partitions = self._generate_columns_partitions(df, out_table, drop, partition_list)
        self._generate_table(out_table, columns, partitions, drop)

        # generate table, get sql list, and write table parallel
        compiled_sql_list = self._dfs2sqls([df], partition_list)
        sql_list = self._write(out_table, columns, partitions, compiled_sql_list)
        logger.info(f'{self.__class__.__name__}:async_df:{out_table},{duration.minute()}')

        # for tqs
        if tqs:
            self.tqs.generate_table(out_table, columns, partitions, drop)
            self.tqs.write(out_table, sql_list)

    def _compile_sql(self, df):
        # remove "Stage 1: " and "SQL compiled: "
        return str(df.compile()).replace('Stage 1: ', '').replace('SQL compiled: ', '').strip()

    def _dfs2sqls(self, dfs, partition_list):
        ret = []
        for df in dfs:
            if partition_list is not None and type(partition_list[0]) == type([]):
                for i in range(1, len(partition_list)):
                    partition_com = {}
                    for j in range(0, len(partition_list[0])):
                        partition_com[partition_list[0][j]] = partition_list[i][j]
                    conditions = True
                    for k, v in partition_com.items():
                        conditions = conditions & (df[k] == v)
                    ret.append({
                        "compile_sql": self._compile_sql(df[conditions]),
                        "partition_com": partition_com
                    })
            else:
                ret.append({
                    "compile_sql": self._compile_sql(df),
                    "partition_com": None
                })
        return ret

    def _generate_columns_partitions(self, df=None, out_table=None, drop=True, partition_list=None):
        pc = partition_list
        if partition_list is not None and type(partition_list[0]) == type([]):
            pc = partition_list[0]
        if drop or not self.dfu.exist_table(out_table):
            cols = list(df.dtypes)
        else:
            # MUST have to get DataFrame correctly
            self.init(input_tables=[out_table], prefix=self.prefix)
            out = self.dfu.get_table(out_table)
            cols = list(out.dtypes)
        columns = []
        partitions = []
        for t in cols:
            to_type = df2dw_conf.get(t.type.name.lower())
            if to_type is None:
                to_type = t.type.name.lower()
            if pc is not None and t.name in pc:
                partitions.append(Partition(name=t, type=to_type))
            else:
                columns.append(Column(name=t, type=to_type))
        return columns, partitions

    def _generate_table(self, table, columns, partitions, drop=True):
        if drop:
            lifecycle = conf['lifecycle']
            self.dfu.delete_table(f'{table}', if_exists=True)
            logger.info(f'{self.__class__.__name__}:Create:{table},lifecycle={lifecycle}')
        else:
            lifecycle = conf['long_lifecycle']
            logger.info(f'{self.__class__.__name__}:Overwrite:{table},lifecycle={lifecycle}')
        if partitions is None or len(partitions) == 0:
            schema = Schema(columns=columns)
        else:
            schema = Schema(columns=columns, partitions=partitions)
        if not self.dfu.exist_table(table):
            self.dfu.create_table(table, schema, lifecycle=lifecycle)

    def _write(self, table, columns, partitions, compiled_sql_list):
        # 1. no partition; 2. dynamic partition; 3. static partitions
        duration = Duration()
        sql_list = []
        for cp in compiled_sql_list:
            compile_sql = cp['compile_sql']
            partition_com = cp['partition_com']
            sql = self.prefix
            if partitions is None or len(partitions) == 0:
                sql += f'''INSERT OVERWRITE table {table} 
                          select {','.join([c.name.name for c in columns])}
                          from ({compile_sql});\n'''
            elif partition_com is None:
                sql += "set hive.exec.dynamic.partition=true; set hive.exec.dynamic.partition.mode=nonstrict;\n"
                sql += f'''INSERT OVERWRITE table {table} PARTITION ({','.join([c.name.name for c in partitions])})
                          select {','.join([c.name.name for c in columns] + [c.name.name for c in partitions])}
                          from ({compile_sql});\n'''
            else:
                logger.info(f'static:table={table},partition_com={partition_com}')
                static_partition = f'''{",".join([f'{k}="{v}"' for k, v in partition_com.items()])}'''
                sql += f'''INSERT OVERWRITE table {table} PARTITION({static_partition})
                          select {','.join([c.name.name for c in columns])}
                          from ({compile_sql})'''
            sql_list.append(sql)
        # step 3: parallel run
        # self._run_sql(ret)
        self.prefix = ''
        logger.info(f'{self.__class__.__name__}:write:{table},{len(sql_list)} sql,{duration.minute()}')
        return sql_list
