# -*- coding: utf-8 -*-
import multiprocessing
import time

from plbrdidge.core.config import conf, df2dw_conf
from plbrdidge.utils.duration import Duration
from plbrdidge.utils.logger import logger
from plbrdidge.utils.tqs_utils import TqsUtils


class Tqs:
    def __init__(self):
        self.prefix = ''  # 'set tqs.query.engine.type=sparkcli;\n'
        self.tqsu = TqsUtils() if conf['tqs'] is None else conf['tqs']

    def _run_sql_list(self, sql_list, table_list=None, return_flag=False, run_sql=True):
        duration = Duration()
        sql_list = self.tqsu.replace(sql_list)
        if not run_sql:
            self._flush(sql_list)
        pool = multiprocessing.Pool(processes=(len(sql_list) if len(sql_list) < 8 else 8))
        result_async = []
        for i in range(len(sql_list)):
            if table_list is None or len(table_list) == 0:
                result_async.append(pool.apply_async(self._run_sql, args=(sql_list[i], None, return_flag, run_sql)))
            else:
                result_async.append(
                    pool.apply_async(self._run_sql, args=(sql_list[i], table_list[i], return_flag, run_sql)))
        results = [None if r is None else r.get() for r in result_async]
        logger.info(f'{self.__class__.__name__}:run_sql_list:{len(sql_list)} sql {duration.minute()}')
        return results

    def _run_sql(self, sql, table=None, return_flag=False, run_sql=True):
        duration = Duration()
        logger.info(f'{self.__class__.__name__}:BeforeTQS:tqs={conf["tqs"].queue},sql={sql}')
        if run_sql:
            job = self.tqsu.execute_query(query=sql)
            if job.is_success():
                if return_flag:
                    result = job.get_result()
                    # logger.info(f"table={table},data={result.fetch_all_data()}")  # 返回查询结果 List[List[Text]]
                    # logger.info(f"table={table},url={result.result_url}")  # 查询结果的 csv 下载链接
                    return result.fetch_all_data()
            else:
                # 任务运行异常，输出相关日志
                logger.error(job.analysis_error_message())
                logger.error(job.query_error_url)
                logger.error(job.query_log_url)
                logger.error(job.tracking_urls)
        logger.info(f'{self.__class__.__name__}:AfterTQS:tqs={conf["tqs"].queue},'
                    f'job_id={job.id if run_sql else None},sql={sql}')
        logger.info(f'{self.__class__.__name__}:run_sql:1 sql {duration.minute()}')

    def cancel(self, job_id):
        logger.info(f'{self.__class__.__name__}:tqs={conf["tqs"].queue},cancelJob={job_id}')
        self.tqsu.cancel_query(job_id)

    def _drop_table(self, table):
        sql = str(self.prefix)
        sleep_time = 2  # after create table, we need to sleep {sleep_time} time, then we can drop table with privilege
        logger.info(f'{self.__class__.__name__}:CreateTableThenSleep:{sleep_time}s')
        time.sleep(sleep_time)
        sql += f'''DROP TABLE IF EXISTS {table};\n'''
        return sql

    def _create_table(self, table, columns, partitions, drop=True):
        sql = str(self.prefix)
        # get the cols, partitions
        cols, partitions_str = [], ''
        for t in columns:
            to_type = df2dw_conf.get(t.name.type.name.lower())
            if to_type is None:
                to_type = t.name.type.name.lower()
            cols.append(f'''\n                    `{t.name.name}` {to_type} COMMENT'{t.name.name}' ''')
        if partitions is not None and len(partitions) > 0:
            pars = []
            for t in partitions:
                to_type = df2dw_conf.get(t.name.type.name.lower())
                if to_type is None:
                    to_type = t.name.type.name.lower()
                pars.append(f'''`{t.name.name}` {to_type} COMMENT '{t.name.name}' ''')
            partitions_str += f'''PARTITIONED BY ({','.join([t for t in pars])})'''
        if drop:
            lifecycle = conf['lifecycle']
        else:
            lifecycle = conf['long_lifecycle']

        # create + drop + create, reason: (1) Dorado cannot delete table is name not exist. (2) keep correct logic if create same table with different schema
        sql += f'''CREATE  TABLE IF NOT EXISTS {table} ({','.join([c for c in cols])} 
                        ) COMMENT '{table}' {partitions_str}
                        ROW     FORMAT SERDE 'org.apache.hadoop.hive.ql.io.parquet.serde.ParquetHiveSerDe' WITH SERDEPROPERTIES ('serialization.format'='1')
                        STORED AS INPUTFORMAT 'org.apache.hadoop.hive.ql.io.parquet.MapredParquetInputFormat' OUTPUTFORMAT 'org.apache.hadoop.hive.ql.io.parquet.MapredParquetOutputFormat'
                        TBLPROPERTIES (
                            'is_starred'='false',
                            'is_core'='false',
                            'parquet.compression'='zstd',
                            'ttl'='{lifecycle}'
                        );\n''';
        return sql

    def _drop_try_again(self, table, retries=1):
        # retries=30 X 2 seconds
        if retries > 30: return
        try:
            logger.info(f"drop_table:table={table},retries={retries}")
            self._run_sql_list(sql_list=[self._drop_table(table)], table_list=[table])
        except:
            self._drop_try_again(table, retries + 1)

    def generate_table(self, table, columns, partitions, drop=True):
        duration = Duration()
        # create + drop + create, reason: (1) create first: due to Dorado need to check table is available first; (2) run two times will give the same results
        self._run_sql_list(sql_list=[self._create_table(table, columns, partitions, drop)], table_list=[table])
        if drop:
            self._drop_try_again(table)
        else:
            sleep_time = 10
            time.sleep(sleep_time)  # Dorado: wait for createTable take effects, e.g., for permanent tables
            logger.info(f'{self.__class__.__name__}:NoDropThenSleep:{sleep_time}s')
        self._run_sql_list(sql_list=[self._create_table(table, columns, partitions, drop)], table_list=[table])
        logger.info(f'{self.__class__.__name__}:generate_table:{table},{duration.minute()}')

    def write(self, table, sql_list):
        # _write 5 tables into 1 table with partitions
        duration = Duration()
        self._run_sql_list(sql_list=sql_list, table_list=[table] * len(sql_list), run_sql=conf['run_sql'])
        logger.info(f'{self.__class__.__name__}:write:{table},{len(sql_list)} sql {duration.minute()}')
        return sql_list

    def _flush(self, sql_list):
        with open(f"{conf['flush_file']}", "a") as f:
            for sql in sql_list:
                f.write(sql + "\n")
            f.close()

    def _static_partition(self, table):
        sql = f'select * from {table};'
        return self._run_sql(sql, table, return_flag=True, run_sql=True)
