# -*- coding: utf-8 -*-
import argparse
from datetime import datetime
from pathlib import Path

from plbrdidge.core.config import conf, odps_conf
from plbrdidge.utils.dataframe_utils import DataFrameUtils
from plbrdidge.utils.logger import logger, add_log_file, add_ch
from plbrdidge.utils.tqs_utils import TqsUtils
from plbrdidge.utils.udf_utils import UdfUtils


def init_setting(dc='va', update_live=False, run_sql=True, access_id=None, access_key=None, path=None):
    app_path = Path(__file__).parent.parent.absolute() if path is None else path
    Path(f"{app_path}/application/").mkdir(parents=True, exist_ok=True)
    Path(f"{app_path}/application/sql").mkdir(parents=True, exist_ok=True)
    Path(f"{app_path}/application/log").mkdir(parents=True, exist_ok=True)
    add_ch(logger)
    add_log_file(logger, path=app_path)

    conf['dc'] = dc if dc is not None else 'va'
    conf['update_live'] = update_live
    conf['run_sql'] = run_sql
    if conf.get('flush_file') is None:
        conf['flush_file'] = f"{app_path}/application/sql/sql_{datetime.now().strftime('%Y%m%d-%H:%M:%S')}.txt"
    if conf.get('df') is None:
        odps_conf['access_id'] = access_id
        odps_conf['access_key'] = access_key
        conf['df'] = DataFrameUtils()
    if conf.get('tqs') is None:
        conf['tqs'] = TqsUtils(queue=dc)
    if conf.get('udf') is None:
        conf['udf'] = UdfUtils()
    logger.info(f"init_setting")
    for k, v in conf.items():
        logger.info(f"{k}:{v}")


def tmp_table(t, s='sources', use_suffix=True, pns_flag=False):
    ret = conf["project_pns"] if pns_flag else conf["project"]
    ret += f'.{conf["prefix"]}{s.lower()}_{t}'
    ret += ('_' + conf['suffix']) if use_suffix else ''
    return ret + ('_live' if conf['update_live'] else '_prelive')


def permanent_table(s='sources', pns_flag=False):
    ret = conf["project_pns"] if pns_flag else conf["project"]
    ret += f'.location_{s.lower()}'
    return ret + ('_live' if conf['update_live'] else '_prelive')


def check(name):
    df = conf['df'].get_table(name)
    logger.info(f'\n{name} = {df.dtypes}')
    logger.info(f'{df.head(100)}')


def cache(df, name, partitions=None, display=False, lifecycle=conf['lifecycle']):
    if partitions is not None:
        df.persist(name, partitions=partitions, odps=o, lifecycle=lifecycle, priority=2)
    else:
        df.persist(name, odps=o, drop_table=True, lifecycle=lifecycle, priority=2)
    logger.info(f'cache:{name}')
    if display:
        check(name)


def dt_to_ms(dt):
    delta = dt - datetime.datetime.utcfromtimestamp(0)
    return int(delta.total_seconds() * 1000)


def dt_from_ms(ms):
    return datetime.datetime.utcfromtimestamp(ms / 1000.0)


def utf(t):
    return t.decode('utf-8') if type(t) == bytes else t


def utf_decode(df):
    for col in df.columns:
        df[col] = df[col].apply(lambda x: utf(x))
        df[col] = df[col].apply(lambda x: '' if x == 'nan' else x)
    return df


def ds_delta(ds, delta=1, pattern='%Y%m%d'):
    dt = datetime.datetime.strptime(ds, pattern) + datetime.timedelta(days=delta)
    return dt.strftime(pattern)


def instance_status(inst):
    if not inst.is_successful():
        logger.info(f'InstanceFailure:{inst},address:{inst.get_logview_address()}')


def parse():
    parser = argparse.ArgumentParser(description="Script the parameters from CMD")
    parser.add_argument("--dc", required=True, type=str, default='va')
    parser.add_argument("--update_live", required=True, type=str, choices=('True', 'False'), default=False)
    parser.add_argument("--run_sql", required=True, type=str, choices=('True', 'False'), default=True)
    parser.add_argument("--access_id", required=True, type=str)
    parser.add_argument("--access_key", required=True, type=str)
    args = parser.parse_args()

    # get parameters
    dc = args.dc
    update_live = args.update_live == 'True'
    run_sql = args.run_sql == 'True'
    access_id = args.access_id
    access_key = args.access_key
    logger.info(
        f"dc={dc},update_live={update_live},run_sql={run_sql},access_id={access_id},access_key={access_key}")
    return dc, update_live, run_sql, access_id, access_key
